module simple_axi_memiface_32
(
  input  clk,
  input  reset,
  input [32-1 : 0] data_address,
  input [32-1 : 0] data_din,
  output [32-1 : 0] data_dout,
  output [32-1 : 0] read_result,
  input  data_we,
  input  data_oe,
  output [32-1 : 0] data_length,
  output  busy,
  input  forbid,
  output [32-1 : 0] axi_reader_ARADDR,
  output [8-1 : 0] axi_reader_ARLEN,
  output  axi_reader_ARVALID,
  input  axi_reader_ARREADY,
  output [3-1 : 0] axi_reader_ARSIZE,
  output [2-1 : 0] axi_reader_ARBURST,
  output [4-1 : 0] axi_reader_ARCACHE,
  output [3-1 : 0] axi_reader_ARPROT,
  input [32-1 : 0] axi_reader_RDATA,
  input [2-1 : 0] axi_reader_RRESP,
  input  axi_reader_RLAST,
  input  axi_reader_RVALID,
  output  axi_reader_RREADY,
  output [32-1 : 0] axi_writer_AWADDR,
  output [8-1 : 0] axi_writer_AWLEN,
  output  axi_writer_AWVALID,
  output [3-1 : 0] axi_writer_AWSIZE,
  output [2-1 : 0] axi_writer_AWBURST,
  output [4-1 : 0] axi_writer_AWCACHE,
  output [3-1 : 0] axi_writer_AWPROT,
  input  axi_writer_AWREADY,
  output [32-1 : 0] axi_writer_WDATA,
  output  axi_writer_WLAST,
  output  axi_writer_WVALID,
  input  axi_writer_WREADY,
  output [4-1 : 0] axi_writer_WSTRB,
  input [2-1 : 0] axi_writer_BRESP,
  input  axi_writer_BVALID,
  output  axi_writer_BREADY
);

  wire  clk_sig;
  wire  reset_sig;
  wire [32-1 : 0] data_address_sig;
  wire [32-1 : 0] data_din_sig;
  reg [32-1 : 0] data_dout_sig = 0;
  reg [32-1 : 0] read_result_sig = 0;
  wire  data_we_sig;
  wire  data_oe_sig;
  wire [32-1 : 0] data_length_sig;
  wire  busy_sig;
  wire  forbid_sig;
  reg [32-1 : 0] axi_reader_ARADDR_sig = 0;
  reg [8-1 : 0] axi_reader_ARLEN_sig = 0;
  reg  axi_reader_ARVALID_sig = 1'b0;
  wire  axi_reader_ARREADY_sig;
  wire [3-1 : 0] axi_reader_ARSIZE_sig;
  wire [2-1 : 0] axi_reader_ARBURST_sig;
  wire [4-1 : 0] axi_reader_ARCACHE_sig;
  wire [3-1 : 0] axi_reader_ARPROT_sig;
  wire [32-1 : 0] axi_reader_RDATA_sig;
  wire [2-1 : 0] axi_reader_RRESP_sig;
  wire  axi_reader_RLAST_sig;
  wire  axi_reader_RVALID_sig;
  reg  axi_reader_RREADY_sig = 1'b0;
  reg [32-1 : 0] axi_writer_AWADDR_sig = 0;
  reg [8-1 : 0] axi_writer_AWLEN_sig = 0;
  reg  axi_writer_AWVALID_sig = 1'b0;
  wire [3-1 : 0] axi_writer_AWSIZE_sig;
  wire [2-1 : 0] axi_writer_AWBURST_sig;
  wire [4-1 : 0] axi_writer_AWCACHE_sig;
  wire [3-1 : 0] axi_writer_AWPROT_sig;
  wire  axi_writer_AWREADY_sig;
  reg [32-1 : 0] axi_writer_WDATA_sig = 0;
  reg  axi_writer_WLAST_sig = 1'b0;
  reg  axi_writer_WVALID_sig = 1'b0;
  wire  axi_writer_WREADY_sig;
  wire [4-1 : 0] axi_writer_WSTRB_sig;
  wire [2-1 : 0] axi_writer_BRESP_sig;
  wire  axi_writer_BVALID_sig;
  reg  axi_writer_BREADY_sig = 1'b0;

  reg  read_state_busy = 1'b0;
  reg  write_state_busy = 1'b0;
  wire  tmp_0001;
  wire  tmp_0002;
  localparam write_seq_IDLE = 32'd0;
  localparam write_seq_s0 = 32'd1;
  localparam write_seq_s1 = 32'd2;
  reg [31:0] write_seq = write_seq_IDLE;
  reg [31:0] write_seq_prev = write_seq_IDLE;
  reg signed [32-1 : 0] write_seq_delay = 0;
  wire  tmp_0003;
  wire  tmp_0004;
  wire  tmp_0005;
  wire  tmp_0006;
  localparam read_seq_IDLE = 32'd0;
  localparam read_seq_s0 = 32'd1;
  localparam read_seq_s1 = 32'd2;
  reg [31:0] read_seq = read_seq_IDLE;
  reg [31:0] read_seq_prev = read_seq_IDLE;
  reg signed [32-1 : 0] read_seq_delay = 0;
  wire  tmp_0007;
  wire  tmp_0008;


  assign clk_sig = clk;
  assign reset_sig = reset;
  assign data_address_sig = data_address;
  assign data_din_sig = data_din;
  assign data_dout = data_dout_sig;
  always @(posedge clk) begin
    if(reset == 1'b1) begin
      data_dout_sig <= 0;
    end else begin
      if (read_seq == read_seq_s1) begin
        data_dout_sig <= axi_reader_RDATA_sig;
      end
    end
  end

  assign read_result = read_result_sig;
  always @(posedge clk) begin
    if(reset == 1'b1) begin
      read_result_sig <= 0;
    end else begin
      if (read_seq == read_seq_s1) begin
        read_result_sig <= axi_reader_RDATA_sig;
      end
    end
  end

  assign data_we_sig = data_we;
  assign data_oe_sig = data_oe;
  assign data_length = data_length_sig;
  assign data_length_sig = 32'sh7fffffff;

  assign busy = busy_sig;
  assign busy_sig = tmp_0002;

  assign forbid_sig = forbid;
  assign axi_reader_ARADDR = axi_reader_ARADDR_sig;
  always @(posedge clk) begin
    if(reset == 1'b1) begin
      axi_reader_ARADDR_sig <= 0;
    end else begin
      if (read_seq == read_seq_IDLE) begin
        axi_reader_ARADDR_sig <= data_address_sig;
      end
    end
  end

  assign axi_reader_ARLEN = axi_reader_ARLEN_sig;
  always @(posedge clk) begin
    if(reset == 1'b1) begin
      axi_reader_ARLEN_sig <= 0;
    end else begin
      if (read_seq == read_seq_IDLE) begin
        axi_reader_ARLEN_sig <= 8'sh00;
      end
    end
  end

  assign axi_reader_ARVALID = axi_reader_ARVALID_sig;
  always @(posedge clk) begin
    if(reset == 1'b1) begin
      axi_reader_ARVALID_sig <= 1'b0;
    end else begin
      if (read_seq == read_seq_IDLE) begin
        axi_reader_ARVALID_sig <= data_oe_sig;
      end else if (read_seq == read_seq_s0) begin
        axi_reader_ARVALID_sig <= tmp_0007;
      end
    end
  end

  assign axi_reader_ARREADY_sig = axi_reader_ARREADY;
  assign axi_reader_ARSIZE = axi_reader_ARSIZE_sig;
  assign axi_reader_ARSIZE_sig = 3'b010;

  assign axi_reader_ARBURST = axi_reader_ARBURST_sig;
  assign axi_reader_ARBURST_sig = 2'b01;

  assign axi_reader_ARCACHE = axi_reader_ARCACHE_sig;
  assign axi_reader_ARCACHE_sig = 4'h3;

  assign axi_reader_ARPROT = axi_reader_ARPROT_sig;
  assign axi_reader_ARPROT_sig = 3'b000;

  assign axi_reader_RDATA_sig = axi_reader_RDATA;
  assign axi_reader_RRESP_sig = axi_reader_RRESP;
  assign axi_reader_RLAST_sig = axi_reader_RLAST;
  assign axi_reader_RVALID_sig = axi_reader_RVALID;
  assign axi_reader_RREADY = axi_reader_RREADY_sig;
  always @(posedge clk) begin
    if(reset == 1'b1) begin
      axi_reader_RREADY_sig <= 1'b0;
    end else begin
      if (read_seq == read_seq_IDLE) begin
        axi_reader_RREADY_sig <= 1'b1;
      end
    end
  end

  assign axi_writer_AWADDR = axi_writer_AWADDR_sig;
  always @(posedge clk) begin
    if(reset == 1'b1) begin
      axi_writer_AWADDR_sig <= 0;
    end else begin
      if (write_seq == write_seq_IDLE) begin
        axi_writer_AWADDR_sig <= data_address_sig;
      end
    end
  end

  assign axi_writer_AWLEN = axi_writer_AWLEN_sig;
  always @(posedge clk) begin
    if(reset == 1'b1) begin
      axi_writer_AWLEN_sig <= 0;
    end else begin
      if (write_seq == write_seq_IDLE) begin
        axi_writer_AWLEN_sig <= 8'sh00;
      end
    end
  end

  assign axi_writer_AWVALID = axi_writer_AWVALID_sig;
  always @(posedge clk) begin
    if(reset == 1'b1) begin
      axi_writer_AWVALID_sig <= 1'b0;
    end else begin
      if (write_seq == write_seq_IDLE) begin
        axi_writer_AWVALID_sig <= data_we_sig;
      end else if (write_seq == write_seq_s0) begin
        axi_writer_AWVALID_sig <= tmp_0003;
      end
    end
  end

  assign axi_writer_AWSIZE = axi_writer_AWSIZE_sig;
  assign axi_writer_AWSIZE_sig = 3'b010;

  assign axi_writer_AWBURST = axi_writer_AWBURST_sig;
  assign axi_writer_AWBURST_sig = 2'b01;

  assign axi_writer_AWCACHE = axi_writer_AWCACHE_sig;
  assign axi_writer_AWCACHE_sig = 4'h3;

  assign axi_writer_AWPROT = axi_writer_AWPROT_sig;
  assign axi_writer_AWPROT_sig = 3'b000;

  assign axi_writer_AWREADY_sig = axi_writer_AWREADY;
  assign axi_writer_WDATA = axi_writer_WDATA_sig;
  always @(posedge clk) begin
    if(reset == 1'b1) begin
      axi_writer_WDATA_sig <= 0;
    end else begin
      if (write_seq == write_seq_IDLE) begin
        axi_writer_WDATA_sig <= data_din_sig;
      end
    end
  end

  assign axi_writer_WLAST = axi_writer_WLAST_sig;
  always @(posedge clk) begin
    if(reset == 1'b1) begin
      axi_writer_WLAST_sig <= 1'b0;
    end else begin
      if (write_seq == write_seq_s0) begin
        axi_writer_WLAST_sig <= axi_writer_AWREADY_sig;
      end else if (write_seq == write_seq_s1) begin
        axi_writer_WLAST_sig <= tmp_0004;
      end
    end
  end

  assign axi_writer_WVALID = axi_writer_WVALID_sig;
  always @(posedge clk) begin
    if(reset == 1'b1) begin
      axi_writer_WVALID_sig <= 1'b0;
    end else begin
      if (write_seq == write_seq_s0) begin
        axi_writer_WVALID_sig <= axi_writer_AWREADY_sig;
      end else if (write_seq == write_seq_s1) begin
        axi_writer_WVALID_sig <= tmp_0005;
      end
    end
  end

  assign axi_writer_WREADY_sig = axi_writer_WREADY;
  assign axi_writer_WSTRB = axi_writer_WSTRB_sig;
  assign axi_writer_WSTRB_sig = 4'hf;

  assign axi_writer_BRESP_sig = axi_writer_BRESP;
  assign axi_writer_BVALID_sig = axi_writer_BVALID;
  assign axi_writer_BREADY = axi_writer_BREADY_sig;
  always @(posedge clk) begin
    if(reset == 1'b1) begin
      axi_writer_BREADY_sig <= 1'b0;
    end else begin
      if (write_seq == write_seq_IDLE) begin
        axi_writer_BREADY_sig <= 1'b1;
      end
    end
  end


  assign tmp_0001 = write_state_busy | read_state_busy;
  assign tmp_0002 = tmp_0001 | forbid_sig;
  assign tmp_0003 = ~axi_writer_AWREADY_sig;
  assign tmp_0004 = ~axi_writer_WREADY_sig;
  assign tmp_0005 = ~axi_writer_WREADY_sig;
  assign tmp_0006 = ~axi_writer_WREADY_sig;
  assign tmp_0007 = ~axi_reader_ARREADY_sig;
  assign tmp_0008 = ~axi_reader_RVALID_sig;

  always @(posedge clk) begin
    if(reset == 1'b1) begin
      write_seq <= write_seq_IDLE;
      write_seq_delay <= 32'h0;
    end else begin
      case (write_seq)
        write_seq_IDLE : begin
          if (data_we_sig == 1'b1) begin
            write_seq <= write_seq_s0;
          end
        end
        write_seq_s0 : begin
          if (axi_writer_AWREADY_sig == 1'b1) begin
            write_seq <= write_seq_s1;
          end
        end
        write_seq_s1 : begin
          if (axi_writer_WREADY_sig == 1'b1) begin
            write_seq <= write_seq_IDLE;
          end
        end
      endcase
    end
  end

  always @(posedge clk) begin
    if(reset == 1'b1) begin
      read_seq <= read_seq_IDLE;
      read_seq_delay <= 32'h0;
    end else begin
      case (read_seq)
        read_seq_IDLE : begin
          if (data_oe_sig == 1'b1) begin
            read_seq <= read_seq_s0;
          end
        end
        read_seq_s0 : begin
          if (axi_reader_ARREADY_sig == 1'b1) begin
            read_seq <= read_seq_s1;
          end
        end
        read_seq_s1 : begin
          if (axi_reader_RVALID_sig == 1'b1) begin
            read_seq <= read_seq_IDLE;
          end
        end
      endcase
    end
  end


  always @(posedge clk) begin
    if(reset == 1'b1) begin
      read_state_busy <= 1'b0;
    end else begin
      if (read_seq == read_seq_IDLE) begin
        read_state_busy <= data_oe_sig;
      end else if (read_seq == read_seq_s1) begin
        read_state_busy <= tmp_0008;
      end
    end
  end

  always @(posedge clk) begin
    if(reset == 1'b1) begin
      write_state_busy <= 1'b0;
    end else begin
      if (write_seq == write_seq_IDLE) begin
        write_state_busy <= data_we_sig;
      end else if (write_seq == write_seq_s1) begin
        write_state_busy <= tmp_0006;
      end
    end
  end



endmodule
