library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

entity upl_input_port is
  port (
    UPLGlobalClk : in std_logic;
    UPLReset : in std_logic;
    ready : out std_logic;
    done : in std_logic;
    recv_length : out signed(32-1 downto 0);
    data_length : out signed(32-1 downto 0);
    data_dout : out signed(32-1 downto 0);
    data_address : in signed(32-1 downto 0);
    data_din : in signed(32-1 downto 0);
    data_we : in std_logic;
    data_oe : in std_logic;
    UPLIn_Data : in std_logic_vector(32-1 downto 0);
    UPLIn_En : in std_logic;
    UPLIn_Request : in std_logic;
    UPLIn_Ack : out std_logic
  );
end upl_input_port;

architecture RTL of upl_input_port is

  attribute mark_debug : string;
  attribute keep : string;
  attribute S : string;

  component dualportram
    generic (
  WIDTH : integer := 32;
  DEPTH : integer := 10;
  WORDS : integer := 1024
    );
    port (
      clk : in std_logic;
      reset : in std_logic;
      length : out signed(31 downto 0);
      address : in signed(31 downto 0);
      din : in signed(WIDTH-1 downto 0);
      dout : out signed(WIDTH-1 downto 0);
      we : in std_logic;
      oe : in std_logic;
      address_b : in signed(31 downto 0);
      din_b : in signed(WIDTH-1 downto 0);
      dout_b : out signed(WIDTH-1 downto 0);
      we_b : in std_logic;
      oe_b : in std_logic
    );
  end component dualportram;

  signal UPLGlobalClk_sig : std_logic := '0';
  signal UPLReset_sig : std_logic := '0';
  signal ready_sig : std_logic := '0';
  signal done_sig : std_logic := '0';
  signal recv_length_sig : signed(32-1 downto 0) := (others => '0');
  signal data_length_sig : signed(32-1 downto 0) := (others => '0');
  signal data_dout_sig : signed(32-1 downto 0) := (others => '0');
  signal data_address_sig : signed(32-1 downto 0) := (others => '0');
  signal data_din_sig : signed(32-1 downto 0) := (others => '0');
  signal data_we_sig : std_logic := '0';
  signal data_oe_sig : std_logic := '0';
  signal UPLIn_Data_sig : std_logic_vector(32-1 downto 0) := (others => '0');
  signal UPLIn_En_sig : std_logic := '0';
  signal UPLIn_Request_sig : std_logic := '0';
  signal UPLIn_Ack_sig : std_logic := '0';

  signal local_rdata : signed(32-1 downto 0) := (others => '0');
  signal local_addr : signed(32-1 downto 0) := (others => '0');
  signal local_din : signed(32-1 downto 0) := (others => '0');
  signal local_we : std_logic := '0';
  signal local_oe : std_logic := '0';
  signal U_RAM_clk : std_logic := '0';
  signal U_RAM_reset : std_logic := '0';
  signal U_RAM_length : signed(32-1 downto 0) := (others => '0');
  signal U_RAM_address : signed(32-1 downto 0) := (others => '0');
  signal U_RAM_din : signed(32-1 downto 0) := (others => '0');
  signal U_RAM_dout : signed(32-1 downto 0) := (others => '0');
  signal U_RAM_we : std_logic := '0';
  signal U_RAM_oe : std_logic := '0';
  signal U_RAM_address_b : signed(32-1 downto 0) := (others => '0');
  signal U_RAM_din_b : signed(32-1 downto 0) := (others => '0');
  signal U_RAM_dout_b : signed(32-1 downto 0) := (others => '0');
  signal U_RAM_we_b : std_logic := '0';
  signal U_RAM_oe_b : std_logic := '0';
  signal recv_count : signed(32-1 downto 0) := (others => '0');
  type Type_main is (
    main_IDLE,
    main_RECV_WAIT,
    main_RECV_DATA,
    main_OPERATION  
  );
  signal main : Type_main := main_IDLE;
  signal main_prev : Type_main := main_IDLE;
  signal main_delay : signed(32-1 downto 0) := (others => '0');
  signal tmp_0001 : std_logic := '0';
  signal tmp_0002 : signed(32-1 downto 0) := (others => '0');
  signal tmp_0003 : signed(32-1 downto 0) := (others => '0');
  signal tmp_0004 : signed(32-1 downto 0) := (others => '0');
  signal tmp_0005 : signed(32-1 downto 0) := (others => '0');
  signal tmp_0006 : signed(32-1 downto 0) := (others => '0');
  signal tmp_0007 : std_logic := '0';
  signal tmp_0008 : std_logic := '0';
  signal done_reg : std_logic := '0';
  signal tmp_0009 : std_logic := '0';
  signal tmp_0010 : std_logic := '0';

begin

  UPLGlobalClk_sig <= UPLGlobalClk;
  UPLReset_sig <= UPLReset;
  ready <= ready_sig;
  process(UPLGlobalClk)
  begin
    if rising_edge(UPLGlobalClk) then
      if UPLReset = '1' then
        ready_sig <= '0';
      else
        if main = main_OPERATION then
          ready_sig <= '1';
        else
          ready_sig <= '0';
        end if;
      end if;
    end if;
  end process;

  done_sig <= done;
  recv_length <= recv_length_sig;
  data_length <= data_length_sig;
  data_length_sig <= U_RAM_length;

  data_dout <= data_dout_sig;
  data_dout_sig <= U_RAM_dout;

  data_address_sig <= data_address;
  data_din_sig <= data_din;
  data_we_sig <= data_we;
  data_oe_sig <= data_oe;
  UPLIn_Data_sig <= UPLIn_Data;
  UPLIn_En_sig <= UPLIn_En;
  UPLIn_Request_sig <= UPLIn_Request;
  UPLIn_Ack <= UPLIn_Ack_sig;
  process(UPLGlobalClk)
  begin
    if rising_edge(UPLGlobalClk) then
      if UPLReset = '1' then
        UPLIn_Ack_sig <= '0';
      else
        if main = main_RECV_WAIT then
          UPLIn_Ack_sig <= tmp_0001;
        else
          UPLIn_Ack_sig <= '0';
        end if;
      end if;
    end if;
  end process;


  -- expressions
  tmp_0001 <= not UPLIn_En_sig;
  tmp_0002 <= X"00000001" when UPLIn_En_sig = '1' else X"00000000";
  tmp_0003 <= local_addr + X"00000001";
  tmp_0004 <= tmp_0003 when UPLIn_En_sig = '1' else local_addr;
  tmp_0005 <= recv_count + X"00000001";
  tmp_0006 <= tmp_0005 when UPLIn_En_sig = '1' else recv_count;
  tmp_0007 <= '1' when UPLIn_En_sig = '1' else '0';
  tmp_0008 <= '1' when UPLIn_En_sig = '0' else '0';
  tmp_0009 <= not done_reg;
  tmp_0010 <= done_sig and tmp_0009;

  -- sequencers
  process (UPLGlobalClk)
  begin
    if rising_edge(UPLGlobalClk) then
      if UPLReset = '1' then
        main <= main_IDLE;
        main_delay <= (others => '0');
      else
        case (main) is
          when main_IDLE => 
            main <= main_RECV_WAIT;
          when main_RECV_WAIT => 
            if tmp_0007 = '1' then
              main <= main_RECV_DATA;
            end if;
          when main_RECV_DATA => 
            if tmp_0008 = '1' then
              main <= main_OPERATION;
            end if;
          when main_OPERATION => 
            if tmp_0010 = '1' then
              main <= main_IDLE;
            end if;
          when others => null;
        end case;
        done_reg <= done_sig;
      end if;
    end if;
  end process;


  local_rdata <= U_RAM_dout_b;

  process(UPLGlobalClk)
  begin
    if rising_edge(UPLGlobalClk) then
      if UPLReset = '1' then
        local_addr <= (others => '0');
      else
        if main = main_RECV_WAIT then
          local_addr <= X"00000000";
        elsif main = main_RECV_DATA then
          local_addr <= tmp_0004;
        end if;
      end if;
    end if;
  end process;

  process(UPLGlobalClk)
  begin
    if rising_edge(UPLGlobalClk) then
      if UPLReset = '1' then
        local_din <= (others => '0');
      else
        if main = main_RECV_WAIT then
          local_din <= signed(UPLIn_Data_sig);
        elsif main = main_RECV_DATA then
          local_din <= signed(UPLIn_Data_sig);
        end if;
      end if;
    end if;
  end process;

  process(UPLGlobalClk)
  begin
    if rising_edge(UPLGlobalClk) then
      if UPLReset = '1' then
        local_we <= '0';
      else
        if main = main_RECV_WAIT then
          local_we <= UPLIn_En_sig;
        elsif main = main_RECV_DATA then
          local_we <= UPLIn_En_sig;
        end if;
      end if;
    end if;
  end process;

  U_RAM_clk <= UPLGlobalClk_sig;

  U_RAM_reset <= UPLReset_sig;

  U_RAM_address <= data_address_sig;

  U_RAM_din <= data_din_sig;

  U_RAM_we <= data_we_sig;

  U_RAM_oe <= data_oe_sig;

  U_RAM_address_b <= local_addr;

  U_RAM_din_b <= local_din;

  U_RAM_we_b <= local_we;

  U_RAM_oe_b <= local_oe;

  process(UPLGlobalClk)
  begin
    if rising_edge(UPLGlobalClk) then
      if UPLReset = '1' then
        recv_count <= (others => '0');
      else
        if main = main_RECV_WAIT then
          recv_count <= tmp_0002;
        elsif main = main_RECV_DATA then
          recv_count <= tmp_0006;
        end if;
      end if;
    end if;
  end process;


  inst_U_RAM : dualportram
  generic map(
    WIDTH => 32,
    DEPTH => 10,
    WORDS => 1024
  )
  port map(
    clk => UPLGlobalClk,
    reset => UPLReset,
    length => U_RAM_length,
    address => U_RAM_address,
    din => U_RAM_din,
    dout => U_RAM_dout,
    we => U_RAM_we,
    oe => U_RAM_oe,
    address_b => U_RAM_address_b,
    din_b => U_RAM_din_b,
    dout_b => U_RAM_dout_b,
    we_b => U_RAM_we_b,
    oe_b => U_RAM_oe_b
  );


end RTL;
